{-# LANGUAGE DeriveGeneric, OverloadedStrings #-}
module GEODE.Metadata.Work
  ( Work(..) ) where

import Control.Applicative (Alternative(..))
import Data.Aeson (FromJSON(..), ToJSON(..), withText)
import Data.ByteString.Char8 as ByteString (unpack)
import Data.Csv (FromField(..), ToField(..))
import Data.Char (toLower)
import Data.Text as Text (unpack)
import GHC.Generics (Generic)
import Text.Read (Read(..), get)

data Work = EDdA | LGE | Wikipedia deriving (Eq, Generic, Ord, Show)

readCaseInsensitive :: Alternative m => String -> m Work
readCaseInsensitive = recognize . fmap toLower
  where
    recognize "edda" = pure EDdA
    recognize "lge" = pure LGE
    recognize "wikipedia" = pure Wikipedia
    recognize _ = empty

instance Read Work where
  readPrec = many get >>= readCaseInsensitive

instance FromField Work where
  parseField = readCaseInsensitive . ByteString.unpack

instance ToField Work where
  toField = toField . show

instance ToJSON Work
instance FromJSON Work where
  parseJSON = withText "Work" $ readCaseInsensitive . Text.unpack
